/* Conversion program from ALT4 input format to RASP.ENG format    */
/* Reads a single input file off stdin and writes converted data   */
/* to stdout.                                                      */
/* Author: Mark Johnson                                            */
/* Date:   11/23/94                                                */

/* Note: This program assumes that the input file is in the        */
/* recognized ALT4 format. It has little or no error checking for  */
/* invalid input records. The assumption is currently that all     */
/* masses are in grams and thrust is in pounds. The program will   */
/* abort rather ungracefully if > 64 thrust-time data points are   */
/* included in the input. In that case, it will not produce any    */
/* output. The manufacturer name is summarily truncated to five    */
/* characters because longer names caused problems with the        */
/* RASPINFO data-summary program                                   */
/*                                                                 */
/* Usage:                                                          */
/*                                                                 */
/* convalt4                                                        */
/* ( the program will prompt for input and output files )          */

/* Modification History                                            */
/* Version 1: 11/23/94 - initial version                           */
/*                                                                 */
/* Version 2: 11/25/94 - changes:                                  */
/*  - Added logic to skip output phase if no valid records were    */
/*    parsed. Variable valid1 gets incremented each time a         */
/*    required data element is picked up. If all of the (currently */
/*    nine) required data records are not processed, no output     */
/*    will be produced.                                            */
/*  - Added logic to skip output phase if last thrust-time record  */
/*    parsed did not go to zero thrust. This also means that even  */
/*    though the data is not used, a type 61 mass fraction record  */
/*    is required after the end of the thrust records              */
/*                                                                 */
/* Version 3: 11/28/94 - changes:                                  */
/*  - Corrected the value of LBTONT conversion factor. Was 4.446   */
/*    is now properly 4.448.                                       */
/*  - Added a credit to Rogers Aeroscience for the input data to   */
/*    each conversion output.                                      */
/*                                                                 */
/* Version 4: 12/26/94 - changes:                                  */
/*  - Added front end to parse files by individual filename and    */
/*    output to a selected output file. The output file is always  */
/*    appended onto. It's slower than for loops but more amenable  */
/*    to being ported to graphical-only systems (i.e. Macintosh)   */
/*    Note: NO WILDCARD CAPABILITY ON INPUT FILE NAMES!!!!         */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <ctype.h>
#include <math.h>

#define PROGVER 4 		/* Program Version */
#define MAXLINE 255
#define WORDLEN 20
#define INCHTOMM 25.4
#define LBTONT 4.448
#define VALIDCOUNT 9
#define MINTHRUSTPTS 2

/* Function Prototypes */
char *find_alpha(char *inbuf);
char *find_numeric(char *inbuf);
int ifix(float);

main()
{

	time_t tnow;
	char inbuf[MAXLINE], motortype[WORDLEN], manuf[WORDLEN], units = 'G',
	     infile_name[MAXLINE], outfile_name[MAXLINE];
	char *cp;
	int rtype, diameter, length, delay[5];
	int i, cnt_thrust = 0, valid1 = 0, valid2 = 0;
	float f_diameter, f_length, propmass, loadmass;
	float timept[64], thrustpt[64];
	FILE *infile, *outfile;

	fprintf(stderr, "CONVALT4 Version %d - Convert Rogers Aeroscience motor data to RASP\n",
		PROGVER);
	/* Get input file name. No default, will keep asking. */
	do 
	{
		fprintf(stderr, "Input file name? ");
		inbuf[0] = '\0';
		fgets(inbuf, MAXLINE, stdin);
		sscanf(inbuf, "%s", infile_name);
	} while ( strlen(infile_name) == 0 );
	fprintf(stderr, "%s\n", infile_name);
	
	/* Get output file name. Default is RASP.ENG in current directory. */
	fprintf(stderr, "Output file name? Default is RASP.ENG in current dir: ");
	inbuf[0] = '\0';
	fgets(inbuf, MAXLINE, stdin);
	sscanf(inbuf, "%s", outfile_name);
	if ( strlen(outfile_name) == 0 )
		strcpy(outfile_name, "RASP.ENG");
	fprintf(stderr, "%s\n", outfile_name);
	
	/* Open input and output files. Error out if either fails */
	infile = fopen(infile_name, "r");
	if ( infile == NULL )
	{
		fprintf(stderr, "Abort. Input file %s could not be opened.\n", 
			infile_name);
		return 0;
	}
	outfile = fopen(outfile_name, "a");
	if ( outfile == NULL )
	{
		fprintf(stderr, "Abort. Output file %s could not be opened.\n",
			outfile_name);
		fclose(infile);
		return 0;
	}
	
	/* read .EDX file from standard input until exhausted */
	while ( !feof(infile) )
	{
		fgets(inbuf, MAXLINE, infile);
/*		fprintf(outfile, "%s", inbuf); */

		/* dispatch handling according to record type found */
		sscanf(inbuf, "%d", &rtype);
/*		printf("%d\n", rtype); */
		switch(rtype)
		{
			case 0:
				break;

			/* skip over numeric part of designator and get */
			/* the NAR/Tripoli motor type */
			case 1:
				cp = find_alpha(inbuf);
				sscanf(cp, "%s", motortype);
				valid1++;
				break;

			/* get the manufacturer code */
			case 3:
				cp = find_alpha(inbuf);
				sscanf(cp, "%s", manuf);
				valid1++;
				break;
			
			/* get diameter and convert to mm */
			case 7:
				cp = find_numeric(inbuf);
				sscanf(cp, "%f", &f_diameter);
				f_diameter *= INCHTOMM;
				diameter = ifix(f_diameter);
				valid1++;
				break;
				
			/* get length */
			case 8:
				cp = find_numeric(inbuf);
				sscanf(cp, "%f", &f_length);
				f_length *= INCHTOMM;
				length = ifix(f_length);
				valid1++;
				break;

			/* get weight units - used in type 11, 12 records */
			/* default value is 'G' */
			case 9:
				cp = find_alpha(inbuf);
				sscanf(cp, "%c", &units);
				valid1++;
				break;
			
			/* get propellant mass - future conversion for units values */
			/* other than grams will be done here also */
			case 11:
				cp = find_numeric(inbuf);
				sscanf(cp, "%f", &propmass);
				
				/* convert to kg here for now */
				propmass /= 1000.0;
				valid1++;
				break;
				
			/* get loaded mass - future conversion for units values */
			/* other than grams will be done here also */
			case 12:
				cp = find_numeric(inbuf);
				sscanf(cp, "%f", &loadmass);
				
				/* convert to kg here */
				loadmass /= 1000.0;
				valid1++;
				break;
			
			/* get time delays - informational only for RASP */
			case 19:
				cp = find_numeric(inbuf);
				
				/* cp now points to first of multiple comma delimited */
				/* values. Getting the others is somewhat ugly */
				/* max of 5 values are currently supported */
				sscanf(cp, "%d", &delay[0]);
				
				/* get the rest of the values */
				for(i=1; i<=4; i++)
				{
					/* skip to next delay entry */
					cp = strchr(cp, ',');
					cp++;
					if (cp == NULL)
						break; /* no more */
					sscanf(cp, "%d", &delay[i]);
				}
				valid1++;
				break;
			
			/* get thrust-time curve data points */
			case 20:
				cp = find_numeric(inbuf);
				sscanf(cp, "%f %f", &timept[cnt_thrust], &thrustpt[cnt_thrust]);
				
				/* convert to Nt; ALT4 uses pounds-force */
				thrustpt[cnt_thrust] *= LBTONT;
				cnt_thrust++;
				if (cnt_thrust > 64)
				{
					fprintf(stderr, "Too many thrust points - aborting");
					exit(1);
				}
				valid2++;
				break;
			case 61:
				valid1++;
				break;
				
			default: 
/*				fprintf(stderr, "Record type %d ignored\n", rtype); */
				break;
		}
	}

	/* check for valid data */
	/* verify that all required data was found */
	if ( (valid1 < VALIDCOUNT) || (valid2 < MINTHRUSTPTS) )
	{
		fprintf(stderr, "Required input records missing. No output file produced.\n");
		exit(1);
	}
	/* test to see that the last thrust point went to zero */
	if ( thrustpt[(cnt_thrust-1)] != 0.0 )
	{
		fprintf(stderr, "Incomplete thrust curve. No output file produced.\n");
		exit(1);
	}

	/* output conversion data as a RASP comment line */
	/* convalt4 version and conversion date */
	time(&tnow);
	fprintf(outfile, "; Converted by CONVALT4 Version %d", PROGVER);
	fprintf(outfile, " on %s", ctime(&tnow) ); 
	fprintf(outfile, "; Input data courtesy Rogers Aeroscience.\n");
	fflush(outfile);

	/* make necessary data range and format conversions */
	/* truncate manufacturer to 5 characters */
	manuf[5] = '\0';

	/* blast out header record of RASP entry */
	fprintf(outfile, "%s   %d   %d   %d", motortype, diameter, length, delay[0]);
	for(i=1; i <=4; i++)
	{
		if (delay[i] != 0)
		{
			fprintf(outfile, "-%d", delay[i]);
		}
		 else
		 	break;
	}
	fprintf(outfile, "   %f   %f   %s\n", propmass, loadmass, manuf);
	
	/* blast out thrust-time data records */
	for (i=0; i<64; i++)
	{
		fprintf(outfile, "   %5.3f    %5.3f\n", timept[i], thrustpt[i]);
		if (timept[i] > 0.0 && thrustpt[i] == 0.0)
			break;
	}
	fclose(infile);
	fclose(outfile);
}

/* given a buffer, return a pointer within that buffer that           */
/* points to the first alpha character in the data portion of an ALT4 */
/* record. The input pointer is not altered.                          */

char *find_alpha(char *inbuf)
{
	char *cp;
	
	/* find data portion of record */
	cp = strchr(inbuf, ':');
	cp++; /* go past */
	while( !(isalpha( (int)*cp) ) )
		cp++;
	return cp;

}

/* given a buffer, return a pointer within that buffer that points to */
/* the first numeric character in the data portion of an ALT4 record  */
/* The input pointer is not altered.                                  */

char *find_numeric(char *inbuf)
{
	char *cp;
	
	/* find data portion */
	cp = strchr(inbuf, ':');
	cp++; /* go past */
	while( !(isdigit( (int)*cp) ) )
		cp++;
	return cp;
}

/* fortran-style integer-fix that rounds off correctly */

int ifix(float len)
{
	int ilen;
	
	ilen = (int) len;
	if ( (len - ilen) > 0.500 )
		++ilen;
	return ilen;
}	
