/*
 * RASP89 - Rocket Altitude Simulation Program
 * Based on the simple BASIC program written by
 * Harry Stine in 1979, modified and converted
 * into C by Mark A. Storin, Sept/Oct 1989.
 * Rev 2 by Kent Hoult, June 1990.
 *
 * 10-20-89  modified to allow multiple stages up to 3.
 * 07-01-90  modified to use linear segment engine thrust tables from data file
 * V2.0      smaller time increments, and crude supersonic transition drag.
 *
 * Rev 2.1 by Mark Johnson, July 1990
 * 07-20-90  enforce MAXSTAGE in input routine
 * V2.1	     add notation of burnout for each stage
 * 
 * Rev 2.2 by Mark Johnson, November 1990
 * 11-10-90  disallow 0 or negative number of stages
 * V2.2
 *
 * Rev 2.2.1 by Mark Johnson, January 1992
 * 01-23-92  for output choice 1, use stdout instead of CON - also don't
 * V2.2.1	 close the output stream if it's stdout.
 * 
 * Rev 2.4 by Mark Johnson, June 1992
 * 06-09-92  Add new motors to RASP.ENG.
 *
 * Rev 2.5 by Mark Johnson, January 1993
 * 01-05-93  Mask transonic drag coefficient drop to near zero from Mach 0.8
 * V2.5      to 0.9, by using same algorithm but delaying change to the 
 *           transonic algorithm from Mach 0.8 to Mach 0.9.
 *
 * Rev 2.6 by Chuck Gibke, January-June 1995
 * 01-22-95  Add apogee and delay selection time marks to output,
 * 06-10-95  command line options, splot (shareware screen plot) output format.
 * V2.6      Also obsessive-compulsive program clean-up.
 *
 * Rev 2.61 by Chuck Gibke, August 1995
 * 08-27-95  Fix delta mass for multi-motor stages (from David W. Schultz).
 * V2.61
 *
 * Rev 2.62 by Chuck Gibke, Sept 1995
 * 09-10-95  Fix floating point error on multiple stages.
 * V2.62     Also add ground impact notation.
 */

#define MSDOS  1
/* #define UNIX        1 */

#include <stdlib.h>
#include <stdio.h>
#include <alloc.h>
#include <math.h>
#include <ctype.h>
#include <string.h>

#define TRUE      1
#define FALSE     0
#define MAXENG    3
#define MAXT      64
#define MAXSTAGE  MAXENG 
#define PI        3.1415927
#define G         9.80621
#define IN2M      0.0254
#define GM2LB     0.00220462
#define OZ2KG     0.028349523
#define M2FT      3.280840
#define ROD       0.9144                /* Length of launch rod in meters (36") */
#define MACH0_8   265.168               /* Mach 0.8 in Meters/sec */
#define MACH0_9   298.314               /* Mach 0.9 in Meters/sec */
#define MACH1     331.460               /* Mach 1.0 in Meters/sec */
#define MACH1_2   397.752               /* Mach 1.2 in Meters/sec */
#define MAXPLOT   500                   /* plot points */

/*
 * Global Variables
 */

char    rasp[] = "RASP89 - Rocket Altitude Simulation Program - v2.61";
char    Rasp[] = "RASP89 v2.61:";
char    ptitle[80];

double  f,t,t1,m,m1,a,c,v,x,x_e,v_e,a_e,m_e,d,d_o,c1,r,w;
double  max_a, max_v;
double  rockettwt, irockettwt;
int     nexteng;                        /* next free engine index */
int     destnum;                        /* index into array of output devices */
int     stagenum;                       /* number of stages */

int     verbose = 0;                    /* verbose output (debug) */
int     brief = 0;                      /* brief output (summary only) */

char Mcode[16];                         /* array to hold engine code */
char fname[64];                         /* output file name for simulation */

FILE *stream, *fp, *efile, *fopen();

char efile_name[] = {"rasp.eng"};       /* name of engine database file */

struct engine {
    double t2;         /* thrust duration    (seconds)                 */
    double m2;         /* propellant wt.     (kilograms)               */
    double wt;         /* initial engine wt. (kilograms)               */
    double thrust[MAXT];  /* thrust curve     (Time-sec,Thrust-Newtons) */
    double navg;       /* average thrust     (newtons)                 */
    double ntot;       /* total impulse      (newton-seconds)          */
    double npeak;      /* peak thrust        (newtons)                 */
    int   dia;         /* engine diameter    (millimeters)             */
    int   len;         /* engine length      (millimeters)             */
    int   ndelay;      /* number of delays                             */
    int   delay[8];    /* ejection delays available (sec)              */
    char  mfg[8];      /* manufacturer info                            */
    char  code[8];     /* engine name                                  */
};

struct stage_info {
    int engcnum;      /* index into e_info */
    int engnum;       /* # of engines in stage */
    double ttime;     /* thrust time + previous stage t2 */
    double weight;    /* stage wt w/o engine */
    double totalw;    /* wt of stage w/ engine(s) */
    double maxd;      /* max diameter of stage  */
};

struct stage_info stages[MAXSTAGE];
struct engine e_info[MAXENG];

/***********************************************************************\
 *                                                                     *
 *  load engine data file                                              *
 *                                                                     *
\***********************************************************************/

load_engine(e) 
        int e;
{
        int t, dia, len, more;
        char s[256], *c, sdelay[32];
        double sum,peak, t1, t2, f1, f2;
        double *p;

        for (more=TRUE; more;) {
            if ((c = fgets(s, 256, efile)) != NULL) {
                if (*c != ';') {
                    if (sscanf(s, "%s %d %d %s %lf %lf %s", e_info[e].code, &e_info[e].dia,
                        &e_info[e].len, sdelay, &e_info[e].m2, &e_info[e].wt,
                        e_info[e].mfg) != 7) {
                        printf("\n*** Error - Bad line in rasp.eng\n");
                        exit(1);
                    }
                    more = FALSE;
                }
            } else {
                return (-1);
            }
        }
        t = 0;
        for (more=TRUE; more;) {
            if ((c = fgets(s, 256, efile)) != NULL) {
                if (*c != ';') {
                    if (t >= MAXT) {
                        printf("\n*** Error - Thrust table overflow in rasp.eng\n");
                        exit(1);
                    }
                    if (sscanf(s, "%lf %lf", &e_info[e].thrust[t],
                        &e_info[e].thrust[t+1]) != 2) {
                        printf("\n*** Error - Bad thrust table line in rasp.eng\n");
                        exit(1);
                    }
                    if (e_info[e].thrust[t+1] == 0) {
                        e_info[e].t2 = e_info[e].thrust[t];
                        more = FALSE;
                    } else {
                        t += 2;
                    }
                }
            } else {
                printf("\n*** Error - unexpected EOF while loading thrust curve.\n");
                exit(1);
            }
        }
        t1 = f1 = sum = peak = 0.0;
        p = &e_info[e].thrust[0];

        do {
            t2 = *p++;
            f2 = *p++;
            if (f2 > peak) peak = f2;
            sum += (t2 - t1) * (f1 + f2) * 0.5;
            t1 = t2;
            f1 = f2;
        }  while (f1 != 0.0);

        e_info[e].ntot = sum;
        e_info[e].npeak = peak;
        e_info[e].navg = sum / t1;
        c = sdelay;
        e_info[e].ndelay = 0;

        for (t = 0; t < 7; t++) {
            e_info[e].delay[t] = (int) strtol(c, &c, 10);
            if (*c != '-') {
                e_info[e].delay[t+1] = -1;
                e_info[e].ndelay = t + 1;
                break;
            } else {
                c++;
            }
        }
        return (e);
}

/***********************************************************************\
 *                                                                     *
 *  get motor data                                                     *
 *                                                                     *
\***********************************************************************/

int getmotor (st)
        int st;
{
        int i, match, e;

        match = FALSE;
        e = nexteng++;
        do {
            if (stagenum > 1) printf("Stage %d ", st);
            printf("Motor Code? ");
            scanf("%s", Mcode);
            for (i = 0; i < strlen(Mcode); i++) {
                if (islower(Mcode[i])) Mcode[i] = toupper(Mcode[i]);
            }
            if ((efile = fopen(efile_name,"r")) == NULL) {
                printf("\n*** Error - unable to open engine database %s ***\n", efile_name);
                exit(1);
            }
            while (load_engine(e) >= 0) {
                if (strcmp(Mcode, e_info[e].code) == 0) {
                    match = TRUE;
                    break;
                }
            }
        } while (match == FALSE);
        fclose(efile);
        return(e);
  }


/***********************************************************************\
 *                                                                     *
 *  choose options                                                     *
 *                                                                     *
\***********************************************************************/

choices ()
{
        int     i;
        char    s[8];
        double  time, wt;

        time = 0;
        do {
            printf("Number of stages? ");
            scanf("%d", &stagenum);
        } while ((stagenum > MAXSTAGE) || (stagenum <= 0));

        for (i = 0; i < stagenum; i++) {
            stages[i].engcnum = getmotor(i + 1);
            if (stagenum > 1) {
                printf("Weight of stage %d w/o engine in ounces? ", i+1);
            } else {
                printf("Weight of rocket w/o engine in ounces? ");
            }
            scanf("%lE", &stages[i].weight);
            stages[i].weight *= OZ2KG;
            if (stagenum > 1) {
                printf("Number of engines in stage %d? ", i+1);
            } else {
                printf("Number of engines? ");
            }
            scanf("%d", &stages[i].engnum);
            if (stagenum > 1) {
                printf("Maximum diameter of stage %d in inches? ", i + 1);
            } else {
                printf("Maximum body diameter in inches? ");
            }
            scanf("%lE", &stages[i].maxd);

            stages[i].totalw = stages[i].weight +
                               (e_info[stages[i].engcnum].wt * stages[i].engnum);
            time += e_info[stages[i].engcnum].t2;
            stages[i].ttime = time;
            rockettwt += stages[i].totalw;
        }
        printf("Drag coefficient? ");
        scanf("%lE", &c1);
        destnum = 0;

        do {
            printf("Send data to:\n");
            printf("\t(1) Screen\n");
            printf("\t(2) Printer\n");
            printf("\t(3) Disk file\n");
            printf("\t(4) Screen Plot (splot) file\n");
            printf("Enter #:");
            scanf("%d", &destnum);
#ifdef MSDOS
            if (destnum == 1) {       /* ver 2.2.1 */
                stream = stdout;
                break;
            } else if (destnum == 2) {
                strcpy(fname, "PRN");
            }
#endif
#ifdef UNIX
            if (destnum == 1) {
                strcpy(fname, "/dev/tty");
            } else if (destnum == 2) {
                strcpy(fname, "/dev/lp");
            }
#endif
            else {
                if (destnum != 4) destnum = 3;
                printf("Enter file name:");
                scanf("%s", fname);
            }
            if ((stream = fopen(fname,"w")) == NULL)
                printf("%s cannot be opened.\n",fname);
        } while (stream == NULL);

        if (destnum != 4) {
            fprintf(stream, "%s\n\n", rasp);
            fprintf(stream,
                "Stage        Engine        Bare       Launch    Dia.\n");
            fprintf(stream,
                "Number  (Quantity)  Type   Wt(oz)     Wt(oz)    (in)     Cd \n");
            fprintf(stream,
                "------  ----------------   ------     ------   -----    ----\n");
            wt = rockettwt;

            for (i = 0; i < stagenum; i++) {
                if (stages[i].engnum < 10) {
                    sprintf(s, "  (%d)", stages[i].engnum);
                } else if (stages[i].engnum < 100) {
                    sprintf(s, " (%d)", stages[i].engnum);
                } else {
                    sprintf(s, "(%d)", stages[i].engnum);
                }
                fprintf(stream,
                    "   %d  %s %-12s  %7.2lf    %7.2lf   %5.2lf    %.2lf\n",
                    i + 1, s, e_info[stages[i].engcnum].code,
                    stages[i].weight / OZ2KG, wt / OZ2KG, stages[i].maxd, c1);
                wt -= stages[i].totalw;
            }
            fprintf(stream, "\n");

            if (!brief) {
                fprintf(stream, " %4s   %10s %11s  %15s %9s\n",
                    "Time", "Altitude", "Velocity", "Acceleration", "Weight");
                fprintf(stream, "%4s  %10s  %12s %15s  %9s\n",
                    "(Sec)", "(Feet)", "(Feet/Sec)", "(Feet/Sec^2)", "(Ounces)");
                fprintf(stream, "%4s  %10s %12s %16s %9s\n",
                "------", "--------", "----------", "--------------", "--------");
            }

        } else {

            verbose = 0;
        }

        return (0);
}


/***********************************************************************\
 *                                                                     *
 *  calculate function                                                 *
 *                                                                     *
\***********************************************************************/

calc ()
{
        double  floor();
        int     eti, tt, tti, launched;
        int     burnout;                /* if = 1, stage has reached zero thrust */
        int     stageng;                /* engcnum for current stage */
        int     this_stage;             /* current stage */
        double  old_f;                  /* last engine thrust force from table */
        double  last_f;                 /* last cycle engine thrust force */
        double  old_t;                  /* last engine thrust time    from table */
        double  stagetm;                /* elapsed time for current stage */
        double  stagewt;                /* current wt of this_stage */
        double  diam;                   /* max diam at current time */
        double  t_rod, v_rod, x_rod;    /* launch rod info */
        double  t_apo, x_apo;           /* apogee info */
        double  cc;
        int     i_bo;                   /* burnout index */
        int     inc_bo;                 /* burnout index incremnter */
        double  t_bo[MAXSTAGE];         /* burnout time */
        double  v_bo[MAXSTAGE];         /* burnout vel */
        double  x_bo[MAXSTAGE];         /* burnout alt */
        double  t_gnd, v_gnd, x_gnd;    /* ground impact info */
        int     iplt;                   /* plot point index */
        double  plt[MAXPLOT][3];        /* plot points: t, x, v */
        double  xlen, ylen;             /* plot axis length */
        double  xbias=0.012, ybias=0.03;/* plot legend character size */
        int     delayf;                 /* delay expired */
        double  t_d[7], v_d[7], x_d[7]; /* delay info */
        int     dcnt;                   /* delay in-use index */
        int     apogee;                 /* apogee flag */
        int     run;                    /* loop control */

        double  temp;
        int     i, j, k;

        for (j = 0; j < stagenum; j++) {
            diam = stages[j].maxd;
            if (j > 0) {
                if (diam < stages[j-1].maxd) diam = stages[j-1].maxd;
            }
        }
        launched = 0;
        tti = floor((0.2 / t1) + 0.5);
        tt = 0;
        d = diam * IN2M;
        this_stage = 0;
        stagetm = 0;
        stagewt = stages[this_stage].totalw;
        stageng = stages[this_stage].engcnum;
        m = rockettwt;
        irockettwt = rockettwt;
        c = r * PI * c1 * d * d * 0.125;
        if (verbose > 1) {
            fprintf(stream,"diam = %lf   d = %lf   c = %lf\n", diam, d, c);
            fflush(stream);
        }
        eti = 0;
        old_f = 0.0;
        last_f = 0.0;
        old_t = 0.0;
        max_a = 0.0;
        max_v = 0.0;
        t_rod = 0.0;
        t_gnd = 0.0;
        burnout = 0;
        i_bo = 0;
        t_bo[0] = 0.0;
        t_bo[1] = 0.0;
        t_bo[2] = 0.0;
        dcnt = 0;
        delayf = 0;
        t_apo = 0.0;
        x_apo = 0.0;
        apogee = 0;
        run = TRUE;
        if (destnum != 1) printf("...time %.2lf sec - ignition\n", t);
        plt[0][0] = 0.0;
        plt[0][1] = 0.0;
        plt[0][2] = 0.0;
        iplt = 1;

        while (run) {
            f = 0;
            t += t1;
            stagetm += t1;
            inc_bo = 0;
            if (t > stages[this_stage].ttime) {
                if (this_stage < stagenum-1) {
                    eti = 0;
                    old_f = 0.0;
                    old_t = 0.0;
                    m = rockettwt -= stagewt;
                    this_stage++;
                    stagewt = stages[this_stage].totalw;
                    stageng = stages[this_stage].engcnum;
                    stagetm = 0.0;
                    for (j = this_stage; j < stagenum; j++) {
                        diam = stages[j].maxd;
                        if (j > this_stage) {
                            if (diam < stages[j-1].maxd) diam = stages[j-1].maxd;
                        }
                    }
                    d = diam * IN2M;
                    c = r * PI * c1 * d * d * 0.125;  /* drag form factor */
                    if (verbose > 1) {
                        fprintf(stream,"diam = %lf   d = %lf   c = %lf\n", diam, d, c);
                        fflush(stream);
                    }
                    if (verbose) {
                        fprintf(stream,"Stage %d separation.\n", this_stage);
                        fflush(stream);
                    }
                }
                inc_bo = 1;
            } else if (t <= stages[this_stage].ttime) {
                if ((verbose > 1) && (inc_bo == 1)) {
                    fprintf(stream,">>>stage=%d   t = %lf   ttime = %lf\n",
                            this_stage, t, stages[this_stage].ttime);
                    fprintf(stream,">>>stagetm = %lf   thrust = (%lf,%lf)   stageng=%d\n",
                            stagetm, e_info[stageng].thrust[eti], e_info[stageng].thrust[eti+1], stageng);
                    fflush(stream);
                }
                if (stagetm >= e_info[stageng].thrust[eti]) {
                    old_t = e_info[stageng].thrust[eti];
                    old_f = e_info[stageng].thrust[eti+1];
                    eti += 2;
                }
                if ((verbose) && (inc_bo == 1)) {
                    fprintf(stream, ">>>eti = %d   old_t = %lf    old_f = %lf\n",
                            eti, old_t, old_f);
                    fflush(stream);
                }
                f = e_info[stageng].thrust[eti + 1] - old_f;
                f *= (stagetm - old_t) / (e_info[stageng].thrust[eti] - old_t);
                f += old_f;
                f *= stages[this_stage].engnum;
                m1 = (e_info[stageng].m2 / e_info[stageng].t2) * t1;
                m1 *= stages[this_stage].engnum;
                if ((verbose) && (inc_bo == 1)) {
                    fprintf(stream, ">>>f = %lf    m1 = %lf\n", f, m1);
                    fflush(stream);
                }
                stagewt -= m1;
                rockettwt -= m1;
                m = rockettwt;
                if ((verbose) && (inc_bo == 1)) {
                    fprintf(stream, ">>>stagewt = %lf    rockettwt = %lf   m = %lf\n",
                            stagewt, rockettwt, m);
                    fflush(stream);
                }
            }
            if (verbose && (f > 0.0)) {
                fprintf(stream, "%lf sec: stage %d th=%lf last=%lf old=%lf wt=%lf i=%d\n",
                        t, this_stage, f, last_f, old_f, m, i_bo);
                fflush(stream);
            }

/* delay transition to the transonic drag approximation until Mach 0.9 */
/* There is a region from Mach 0.8 to Mach 0.9 in which the (v < MACH1)*/
/* computation gives effective drag coefficients below subsonic value. */

            if (v < MACH0_9) { /* Crude approximations for MACH 1 Transition */
                cc = c;
            } else if (v < MACH1) {
                cc = c * 2.0 * (v - MACH0_8) / (MACH1 - MACH0_8);
            } else if (v < MACH1_2) {
                cc = c * (2.0 - ((v - MACH1) / (MACH1_2 - MACH1)));
            } else {
                cc = c;
            }

            a = ((f - (cc * v * v)) / m) - G;
            v = v + a * t1;
            x = x + v * t1;

            if (v > 0) {
                launched = 1;
            } else if ((!launched) && (v < 0)) {
                x = v = a = 0;
            } else if (launched && (v < 0)) {
                if (apogee == 0) apogee = 1;
                if (t_apo == 0.0) {
                    t_apo = t;
                    x_apo = x*M2FT;
                }
                if (e_info[stageng].delay[dcnt] < 0) {
                    run = FALSE;
                } else if (x < 0.0) {
                    t_gnd = t;
                    x_gnd = x*M2FT;
                    v_gnd = v*M2FT;
                    run = FALSE;
                }
            }

            if ((t_rod == 0.0) && (x >= ROD)) {
                t_rod = t;
                v_rod = v * M2FT;
                x_rod = x * M2FT;
                if (verbose) {
                    fprintf(stream, "rod time = %lf sec, v=%lf x=%lf\n",
                                    t_rod, v_rod, x_rod);
                    fflush(stream);
                }
            }

            if (a > max_a) max_a = a;
            if (v > max_v) max_v = v;
            tt++;
            if ((f==0.0) && (last_f != 0.0)) {
                burnout = 1;
                t_bo[i_bo] = t;
                v_bo[i_bo] = v*M2FT;
                x_bo[i_bo] = x*M2FT;
                if (verbose) {
                    fprintf(stream, "burnout time = %lf sec, v=%lf x=%lf\n",
                                    t_bo[i_bo], v_bo[i_bo], x_bo[i_bo]);
                    fflush(stream);
                }
            }

            if ((i_bo == stagenum) && (this_stage == (stagenum-1))) {
                if ((t_bo[i_bo-1] > 0.0) && (e_info[stageng].delay[dcnt] >= 0)) {
                    if (t > (t_bo[i_bo-1]+((double)e_info[stageng].delay[dcnt]))) {
                        delayf = TRUE;
                    }
                }
                if ((verbose > 1) && delayf) {
                    fprintf(stream, "time = t+t_bo=%lf dcnt=%d (%lf sec) df=%d\n",
                                    t+t_bo[i_bo-1], dcnt, e_info[stageng].delay[dcnt], delayf);
                    fflush(stream);
                }
            }

            if ((tt == tti) || (burnout) || (delayf) || (apogee == 1) ||
                (t_gnd > 0.0)) {
                if (tt == tti) tt = 0;
                x_e = x * M2FT;
                v_e = v * M2FT;
                a_e = a * M2FT;
                m_e = m / OZ2KG;

                if ((destnum != 4) && (!brief)) {
                    fprintf(stream,"%5.2lf  %10.1lf  %10.1lf     %10.1lf      %8.3lf",
                             t, x_e, v_e, a_e, m_e);
                }

                if (burnout == 1) {
                    if ((destnum != 4) && (!brief)) {
                        if (stagenum > 1) {
                            fprintf(stream, "  <-Stage %d Burnout\n", i_bo+1);
                        } else {
                            fprintf(stream, "  <-Burnout\n");
                        }
                    }
                    if (destnum != 1) printf("...time %.2lf sec - burnout\n", t);
                    burnout = 0;
                    old_f = 0.0;    /* so we won't trip over it again */
                } else if (apogee == 1) {
                    if ((destnum != 4) && (!brief)) {
                        fprintf(stream, "  <-Apogee\n");
                    }
                    if (destnum != 1) printf("...time %.2lf sec - apogee\n", t);
                    apogee = -1;
                } else if (delayf == 1) {
                    if ((destnum != 4) && (!brief)) {
                        fprintf(stream, "  <-Delay (%d sec)\n", e_info[stageng].delay[dcnt]);
                    }
                    if (destnum != 1) printf("...time %.2lf sec - delay (%d sec)\n",
                                            t, e_info[stageng].delay[dcnt]);
                    t_d[dcnt] = t;
                    v_d[dcnt] = v_e;
                    x_d[dcnt] = x_e;
                    delayf = 0;
                    dcnt ++;
                } else if (t_gnd > 0.0) {
                    if ((destnum != 4) && (!brief)) {
                        fprintf(stream, "  <-ground impact!\n");
                    }
                    if (destnum != 1) printf("...time %.2lf sec - ground impact!\n", t);
                } else {
                    if ((destnum != 4) && (!brief)) {
                        fprintf(stream, "\n");
                    }
                }
                fflush(stream);

                if (iplt < MAXPLOT) {
                    plt[iplt][0] = t;
                    plt[iplt][1] = x_e;
                    plt[iplt][2] = fabs(v_e);
                    iplt ++;
                }
            }

            i_bo += inc_bo;
            if (i_bo > stagenum) i_bo = stagenum;
            last_f = f;
        }

/***************************************************\
 *  output summary                                 *
\***************************************************/

        x_e = x * M2FT;
        v_e = v * M2FT;
        a_e = a * M2FT;
        m_e = m / OZ2KG;

        if (destnum != 4) {

	/*-------------------------------------------------*\
     *  printed output                                 *
	\*-------------------------------------------------*/

            fprintf(stream, "\n");
            fprintf(stream, "Maximum altitude attained was %.1lf feet (%.1lf meters).\n",
                    x_apo, x_apo/M2FT);
            fprintf(stream, "Time to peak altitude = %.2lf seconds.\n", t_apo);
            fprintf(stream, "Delay time to peak altitude = %.2lf seconds.\n", t_apo-t_bo[i_bo-1]);
            fprintf(stream, "\n");

            fprintf(stream, "Maximum velocity was %.1lf feet/sec (mach %.1lf).\n", max_v*M2FT, max_v/MACH1);
            fprintf(stream, "Maximum acceleration was %.1lf feet/sec^2.\n\n",
                    max_a * M2FT);

            fprintf(stream, "Event:        __time__   altitude   altitude   velocity\n");
            fprintf(stream, "                (sec)     ft MSL     ft AGL     ft/sec \n");
            fprintf(stream, "----------    --------   --------   --------   --------\n");
            fprintf(stream, "ignition          0.00        0.0        0.0        0.0\n");
            fprintf(stream, "launch rod    %8.2lf   %8.1lf   %8.1lf   %8.1lf\n",
                    t_rod, x_rod, x_rod, v_rod);
            if (stagenum == 1) {
                fprintf(stream, "burnout       %8.2lf   %8.1lf   %8.1lf   %8.1lf\n",
                        t_bo[0], x_bo[0], x_bo[0], v_bo[0]);
            } else {
                for (i=0; i<stagenum; i++) {
                    fprintf(stream, "stage %d       %8.2lf   %8.1lf   %8.1lf   %8.1lf\n",
                            i+1, t_bo[i], x_bo[i], x_bo[i], v_bo[i]);
                }
            }
            fprintf(stream, "apogee        %8.2lf   %8.1lf   %8.1lf   %8.1lf\n",
                    t_apo, x_apo, x_apo, 0.0);
            if (t_gnd > 0.0) {
                fprintf(stream, "gnd impact!   %8.2lf   %8.1lf   %8.1lf   %8.1lf\n",
                        t_gnd, x_gnd, x_gnd, v_gnd);
            }
            fprintf(stream, "\n\n");

            fprintf(stream, "Ejection:      __time__   altitude   altitude   velocity\n");
            fprintf(stream, "                 (sec)     ft MSL     ft AGL     ft/sec \n");
            fprintf(stream, "----------     --------   --------   --------   --------\n");
            for (j=0; j<e_info[stageng].ndelay; j++) {
                if (t_d[j] > 0.0) {
                    fprintf(stream, "%2d sec delay  %8.2lf   %8.1lf   %8.1lf   %8.1lf\n",
                            e_info[stageng].delay[j], t_d[j], x_d[j], x_d[j], v_d[j]);
                }
            }
            if (destnum != 1) fprintf(stream, "");    /* FF */
            fflush(stream);

        } else {

	/*-------------------------------------------------*\
     *  plot output                                    *
	\*-------------------------------------------------*/

            if (stagenum == 1) {
                sprintf(ptitle, "%s %.1lf\" %.2lf oz. airframe, Cd=%.2lf, (%d) %s\n",
                        Rasp, stages[0].maxd, stages[0].weight/OZ2KG, c1,
                        stages[0].engnum, e_info[stages[0].engcnum].code);
            } else {
                sprintf(ptitle, "%s %d Stage %.2lf oz. airframe, Cd=%.2lf\n",
                        Rasp, stagenum, irockettwt/OZ2KG, c1);
            }

            fprintf(stream, "clrdata\n");   /* altitude plot */
            fprintf(stream, "clrplot\n");
            fprintf(stream, "title %s", ptitle);
            fprintf(stream, "xlabel time, sec\n");
            fprintf(stream, "ylabel altitude, ft MSL\n");

            fprintf(stream, "xstart 0\n");
            k = (int) (t + 5.0);
            xlen = (k/5)*5;
            fprintf(stream, "xend %d\n", (k/5)*5);
            fprintf(stream, "xstep 5\n");
            fprintf(stream, "ystart 0\n");
            if (x_apo <= 500.0) {
                j = 100;
                k = (int) (x_apo + 100.0);
                k = (k/j)*j;
            } else if (x_apo <= 3000.0) {
                j = 500;
                k = (int) (x_apo + 500.0);
                k = (k/j)*j;
            } else if (x_apo <= 10000.0) {
                j = 1000;
                k = (int) (x_apo + 1000.0);
                k = (k/j)*j;
            } else {
                j = 5000;
                k = (int) (x_apo + 5000.0);
                k = (k/j)*j;
            }
            ylen = k;
            fprintf(stream, "yend %d\n", k);
            fprintf(stream, "ystep %d\n", j);
            fprintf(stream, "xgrid\n");
            fprintf(stream, "ygrid\n");

            fprintf(stream, "legends on\n");
            fprintf(stream, "legend1 %lf %lf apo\n", t_apo/xlen-xbias, x_apo/ylen+ybias);
            for (i=0; i<i_bo; i++) {
                fprintf(stream, "legend%d %lf %lf BO\n", i+2,
                        t_bo[i]/xlen+xbias, x_bo[i]/ylen-ybias*0.5);
            }
            for (i=0; i<dcnt; i++) {
                fprintf(stream, "legend%d %lf %lf %d\n", i_bo+i+2,
                        t_d[i]/xlen+xbias, x_d[i]/ylen-ybias*1.5, e_info[i_bo-1].delay[i]);
            }

            fprintf(stream, "read %d\n", iplt);
            for (i=0; i<iplt; i++) {
                fprintf(stream, "%lf %lf\n", plt[i][0], plt[i][1]);
            }

            fprintf(stream, "read %d\n", i_bo+dcnt+2);
            fprintf(stream, "%lf %lf\n", 0.0, 0.0);
            fprintf(stream, "%lf %lf\n", t_apo, x_apo);
            for (i=0; i<i_bo; i++) fprintf(stream, "%lf %lf\n", t_bo[i], x_bo[i]);
            for (i=0; i<dcnt; i++) fprintf(stream, "%lf %lf\n", t_d[i], x_d[i]);

            fprintf(stream, "scatter 2\n");
            fprintf(stream, "plot\n");

            fprintf(stream, "clrdata\n");   /* velocity plot */
            fprintf(stream, "clrplot\n");
            fprintf(stream, "title %s", ptitle);
            fprintf(stream, "xlabel time, sec\n");
            fprintf(stream, "ylabel velocity, ft/sec\n");

            fprintf(stream, "xstart 0\n");
            k = (int) (t + 5.0);
            fprintf(stream, "xend %d\n", (k/5)*5);
            fprintf(stream, "xstep 5\n");
            fprintf(stream, "ystart 0\n");
            temp = max_v*M2FT;
            if (temp <= 500.0) {
                j = 100;
                k = (int) (temp + 100.0);
                k = (k/j)*j;
            } else if (temp <= 3000.0) {
                j = 500;
                k = (int) (temp + 500.0);
                k = (k/j)*j;
            } else {
                j = 5000;
                k = (int) (temp + 5000.0);
                k = (k/j)*j;
            }
            ylen = k;
            fprintf(stream, "yend %d\n", k);
            fprintf(stream, "ystep %d\n", j);
            fprintf(stream, "xgrid\n");
            fprintf(stream, "ygrid\n");

            fprintf(stream, "legends on\n");
            fprintf(stream, "legend1 %lf %lf apo\n", t_apo/xlen-xbias, 0.0+ybias);
            for (i=0; i<i_bo; i++) {
                fprintf(stream, "legend%d %lf %lf BO\n", i+2,
                        t_bo[i]/xlen+xbias, v_bo[i]/ylen-ybias*0.5);
            }
            for (i=0; i<dcnt; i++) {
                fprintf(stream, "legend%d %lf %lf %d\n", i_bo+i+2,
                        t_d[i]/xlen+xbias, fabs(v_d[i])/ylen+ybias, e_info[i_bo-1].delay[i]);
            }

            fprintf(stream, "read %d\n", iplt);
            for (i=0; i<iplt; i++) {
                fprintf(stream, "%lf %lf\n", plt[i][0], plt[i][2]);
            }

            fprintf(stream, "read %d\n", i_bo+dcnt);
            for (i=0; i<i_bo; i++) fprintf(stream, "%lf %lf\n", t_bo[i], v_bo[i]);
            for (i=0; i<dcnt; i++) fprintf(stream, "%lf %lf\n", t_d[i], fabs(v_d[i]));

            fprintf(stream, "scatter 2\n");
            fprintf(stream, "plot\n");
            fflush(stream);
        }

        return;
}


/***********************************************************************\
 *                                                                     *
 *  RASP89                                                             *
 *                                                                     *
\***********************************************************************/

main (
        int     argc,
        char    *argv[])
{
        char    ans[3];
        int     again;
        int     i;

/***************************************************\
 *  check command line arguments                   *
\***************************************************/

        for (i=1; i<argc; i++) {
            if (strcmpi(argv[i], "-b") == 0) {
                brief = 1;
            }

            if (strcmpi(argv[i], "-v") == 0) {
                 verbose ++;
            }
        }

        t1 = 0.001;
        r = 1.205;
        printf("\n%s\n\n", rasp);

/***************************************************\
 *  run loop                                       *
\***************************************************/

        do {
            nexteng = 0;
            v = t = m = a = x = 0;
            rockettwt = 0;
            again = 0;

            choices();
            calc();

            if (stream != stdout)       /* ver 2.2.1 - don't close stdout!! */
                fclose(stream);
            printf("\nDo another one (Y/N)? ");
            scanf("%s", ans);
            if(isupper(ans[0])) ans[0] = tolower(ans[0]);
            if (strncmp(ans,"y",1) == 0) again = 1;
        } while (again);

        return (0);
}
