/* Cleanup program for RASP.ENG motor data files.                  */
/* Reads a single input file off stdin and writes converted data   */
/* to stdout.                                                      */
/* Author: Mark Johnson                                            */
/* Date:   12/04/94                                                */

/* Note: This program assumes that the input file is in the        */
/* recognized RASP data format. It does very little error checking */
/* other than to verify that a particular record is one of:        */
/* - a comment record beginning with a ';' as the first nonwhite   */
/*   character.                                                    */
/* - a header record with a total of seven tokens to be read in    */
/* - a thrust curve data record with two tokens to be read in      */
/* No data conversion is performed; all data is read in as string  */
/* tokens and stuffed back out again. Control characters (0x01     */
/* thru 0x1F) are removed from comment lines only.                 */
/*                                                                 */
/* Usage:                                                          */
/*                                                                 */
/*  raspcln                                                        */
/*                                                                 */

/* Modification History                                            */
/* Version 1: 12/04/94 - initial version                           */
/*                                                                 */
/* Version 2: 12/26/94 - add input and output file name capability */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <ctype.h>
#include <math.h>

#define PROGVER 2 		/* Program Version */
#define MAXLINE 255
#define WORDLEN 20
#define HDRCNT 7		/* number of tokens in a header record */
#define THRCNT 2		/* number of tokens in a thrust record */
#define TRUE 1
#define FALSE 0
#define NUL 0x0			/* ASCII nul character */

main()
{
	int i, iscomment, tcount;	
	char inbuf[MAXLINE], infile_name[MAXLINE], outfile_name[MAXLINE];
	char tk1[WORDLEN], tk2[WORDLEN], tk3[WORDLEN], tk4[WORDLEN],
	     tk5[WORDLEN], tk6[WORDLEN], tk7[WORDLEN];
	float timept, thrustpt;
	FILE *infile, *outfile;
	
	fprintf(stderr, "RASPCLN Version %d\n", PROGVER);

	/* Get input file name. Default is: RASP.ENG */
	fprintf(stderr, "Input file name? Default is RASP.ENG in current dir: ");
	inbuf[0] = '\0';
	fgets(inbuf, MAXLINE, stdin);
	sscanf(inbuf, "%s", infile_name);
	if ( strlen(infile_name) == 0 )
		strcpy(infile_name, "RASP.ENG");
	
	/* Get output file name. Default is: RASP.NEW */
	fprintf(stderr, "Output file name? Default is RASP.NEW in current dir: ");
	inbuf[0] = '\0';
	fgets(inbuf, MAXLINE, stdin);
	sscanf(inbuf, "%s", outfile_name);
	if ( strlen(outfile_name) == 0 )
		strcpy(outfile_name, "RASP.NEW");

	/* Open files. If either file cannot be opened, error out */
	if ( (infile = fopen(infile_name, "r")) == NULL )
	{
		fprintf(stderr, "Abort. Input file %s could not be opened.\n",
		        infile_name);
		return 0;
	}
	
	if ( (outfile = fopen(outfile_name, "w")) == NULL )
	{	
		fprintf(stderr, "Abort. Output file %s could not be opened.\n",
			outfile_name);
		fclose(infile);
		return 0;
	}
	
	/* read RASP.ENG file from standard input until exhausted */
	fprintf(outfile, "; File reprocessed by RASPCLN Version %d\n", PROGVER);
	while ( !feof(infile) )
	{
		fgets(inbuf, MAXLINE, infile);

		/* dispatch handling according to record type found */
		iscomment = FALSE;
		/* detect comment lines */
		for (i=0; i<MAXLINE; i++)
		{
			if ( isspace( (int) inbuf[i]) ) 
				continue; /* next char */
			 else /* non-whitespace */
			{
			 	if ( inbuf[i] == ';' )
					iscomment = TRUE;
				 else
				 	iscomment = FALSE;
				break; 
			}
		}
		/* if comment, filter out the control characters in place */
		for (i=0; i<MAXLINE; i++)
		{
			if (( iscntrl( (int) inbuf[i])  ) && ( inbuf[i] != NUL ))
				inbuf[i] = ' ';
		}

		/* output complete comment line w/o further ado */
		/* otherwise, scan in 7 tokens and get the count from sscanf */
		if (iscomment == TRUE)
		{
			fprintf(outfile, "%s\n", inbuf);
			fflush(outfile);
		}
		 else
		{
			tcount = sscanf(inbuf, "%s %s %s %s %s %s %s", 
					tk1, tk2, tk3, tk4, tk5, tk6, tk7);
	
			/* output correctly reformatted line */
			if (tcount == HDRCNT)
			{
				fprintf(outfile, "%s   %s   %s   %s   %s   %s   %s\n", 
					tk1, tk2, tk3, tk4, tk5, tk6, tk7);
			}
			
			if (tcount == THRCNT)
			{
				/* decimalize tk1 and tk2 */
				sscanf(tk1, "%f", &timept);
				sscanf(tk2, "%f", &thrustpt);
				fprintf(outfile, "   %5.3f    %5.3f\n", timept, thrustpt);
			}

		}

		/* purge line buffer */
		for (i=0; i<MAXLINE; i++)
		{
			inbuf[i] = 0x0;
		}

	}
	fprintf(stderr, "RASPCLN conversion complete.\n");
	fclose(infile);
	fclose(outfile);
} 

